// To run this file from terminal put it in a folder, cd to the folder and input
// gcc -o depthf depthfirst.c -lm && ./depthf
// To run successive compiled versions type ./depthf
// To generate a compact png version of the .ppm image output file,
// first make a copy of the .ppm file and call it "test.ppm"
// Then use the following command in Terminal
// sips -s format png test.ppm --out test.png

#include <stdio.h>
#include <math.h>
#include <complex.h>
#include <time.h>

typedef double _Complex arry[2][2];

void mult(arry a,arry b, arry c);
void inv(arry a, arry c);
double _Complex fixp(arry T);

time_t start,end;

int main(void)
{
    FILE *mf;
    const char base[] = "DF";
    char filename [100];
    
    double _Complex ta=1.934141199416-0.036681260008i,tb=2;  
    int spec=2, altroot=0, cnt=0;//spec=2,3 non-free group Tcomm=0,1 altroot sets which square root for tab
    int neck=1,iterate=2; //to aid drawing large whorls with narrow necks using midpoint
    const int levmax=2000,siz=1100,rangeopt=0; //range opt sets escape options on range overflow
    // 1 gives escape 2 gives branch break. For doubly degenerate examples rangeopt=2 for time efficiency

    double ep=0.001,scal=2.2;
    double _Complex tab;
    
    int M[siz][siz],siz2,sizs;
    double _Complex gens[2][2][4];
    double _Complex word[3][3][levmax+1];
    int tags[levmax+1];
    int state[levmax+1];
  
    double dif;
    
    int FSA[19][4]={{1,2,3,4},{1,5,-1,4},{7,2,8,-1},{-1,2,3,6},{9,-1,10,4},{7,2,11,-1},{12,-1,10,4},{1,5,-1,13},{-1,2,3,14},{1,15,-1,4},{-1,16,3,4},{-1,2,3,17},{1,18,-1,4},{9,-1,-1,4},{-1,-1,10,4},{7,2,-1,-1},{-1,2,8,-1},{-1,-1,10,4},{7,2,-1,-1}};
    
    double _Complex fix[3][4];
    int i,j,co,rd,gr,bl;
    double _Complex p,q,z0,z,Q,tC,R,zo;
    arry a,b,ab,A,B,h;
    arry *iv, *mu,h1,h2,h3;
    int stfl,lev,fl,plt,stps;
    double _Complex oldpoint,stpt,newpoint,midpoint;
    double x;
    
    sprintf(filename, "%s %e %e %e %e %d %e %e.ppm",base,creal(ta),cimag(ta),creal(tb),cimag(tb),levmax,ep,scal);
    mf=fopen(filename,"w");
    time (&start);
    ep=ep/scal;
    for (i=0;i<siz;i++)
        for (j=0;j<siz;j++)
            M[i][j]=255;
    siz2=siz/2;
    sizs=(int)siz2*10.0/11;
    //grandma's recipe
    if (spec==0)
    {
        p=-ta*tb;
        q=ta*ta+tb*tb;
        if (altroot==0)
            tab=(-p-csqrt(p*p-4*q))/2;
        else
            tab=(-p+csqrt(p*p-4*q))/2;
        z0=((tab-2)*tb)/(tb*tab-2*ta+2*I*tab);
        ab[0][0]=tab/2;
        ab[0][1]=(tab-2)/(2*z0);
        ab[1][0]=(tab+2)*z0/2;
        ab[1][1]=tab/2;
        b[0][0]=(tb-2*I)/2;
        b[0][1]=tb/2;
        b[1][0]=tb/2;
        b[1][1]=(tb+2*I)/2;
    }
    else
    {
        p=-ta*tb;
	if (spec==2)
        	q=ta*ta+tb*tb-2;
        else
                q=ta*ta+tb*tb-1-2;
        if (altroot==0)
            tab=(-p-csqrt(p*p-4*q))/2;
        else
            tab=(-p+csqrt(p*p-4*q))/2;
        tC=ta*ta+tb*tb+tab*tab-ta*tb*tab-2; //Grandma's three parameter double alarm
        Q=csqrt(2-tC);
        if ((cabs(tC+I*Q*csqrt(tC+2))>=2+1e-15))
            R=csqrt(tC+2);
        else
            R=-csqrt(tC+2);
        zo=(tab-2)*(tb-R)/(tb*tab-2*ta+1i*Q*tab);
        a[0][0]=ta/2;
        a[0][1]=(ta*tab-2*tb+2*I*Q)/((2*tab+4)*zo);
        a[1][0]=(ta*tab-2*tb-2*I*Q)*zo/(2*tab-4);
        a[1][1]=ta/2;
        b[0][0]=(tb-I*Q)/2;
        b[0][1]=(tb*tab-2*ta-I*Q*tab)/((2*tab+4)*zo);
        b[1][0]=(tb*tab-2*ta+I*Q*tab)*zo/(2*tab-4);
        b[1][1]=(tb+I*Q)/2;
        
    }
    inv(b,B);
    if (spec==0)
        mult(ab,B,a);
    inv(a,A);
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][0]=a[i][j];
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][1]=b[i][j];
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][2]=A[i][j];
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][3]=B[i][j];
    mult(b,a,h1); mult(A,h1,h2); mult(B,h2,h3); fix[0][0]=fixp(h3);
    mult(A,b,h1); mult(B,h1,h2); mult(a,h2,h3); fix[0][1]=fixp(h3);
    mult(B,A,h1); mult(a,h1,h2); mult(b,h2,h3); fix[0][2]=fixp(h3);
    mult(a,B,h1); mult(b,h1,h2); mult(A,h2,h3); fix[0][3]=fixp(h3);
    fix[1][0]=fixp(a); fix[1][1]=fixp(b); fix[1][2]=fixp(A); fix[1][3]=fixp(B);
    mult(B,a,h1); mult(A,h1,h2); mult(b,h2,h3); fix[2][0]=fixp(h3);
    mult(a,b,h1); mult(B,h1,h2); mult(A,h2,h3); fix[2][1]=fixp(h3);
    mult(b,A,h1); mult(a,h1,h2); mult(B,h2,h3); fix[2][2]=fixp(h3);
    mult(A,B,h1); mult(b,h1,h2); mult(a,h2,h3); fix[2][3]=fixp(h3);
    lev=0;
    tags[0]=0;
    state[0]=0;
    for (i=0;i<2;i++) for (j=0;j<2;j++) word[i][j][0]=gens[i][j][0];
    
    while (~((lev==-1)&&(tags[0]==1)))
    {
        fl=0;
        if (spec==2)
            if (state[lev]==-1)
                fl=1;
        while (fl==0)
        {
            z=fix[0][tags[lev]];
            newpoint=(word[0][0][lev]*z+word[0][1][lev])/(word[1][0][lev]*z+word[1][1][lev]);
            z=fix[2][tags[lev]];
            oldpoint=(word[0][0][lev]*z+word[0][1][lev])/(word[1][0][lev]*z+word[1][1][lev]);
            if (neck>0)
            {
                z=fix[1][tags[lev]];
                midpoint=(word[0][0][lev]*z+word[0][1][lev])/(word[1][0][lev]*z+word[1][1][lev]);
            }
            plt=0;
            x=scal*cabs(newpoint);
            if (rangeopt>0)
                if (x>10) //Chaotic instability
            {
                if (rangeopt==1)
                {
                    printf("exiting...\n");
                    return(0);
                }
                if (rangeopt==2)
                {
                    printf(".");
                    cnt=cnt+1;
                    if (cnt>200)
                    {
                        printf("\n");
                        cnt=0;
                    }
                    fl=1;
                    break;
                }
            }
            if (neck>0)
            {
                if ((lev==levmax) || ((cabs(newpoint-midpoint)<ep)&&(cabs(midpoint-oldpoint)<ep)))
                {
                    if (x>1.5) //Limit set off screen
                    {
                        fl=1;
                        break;
                    }
                    plt=1;
                }
            }
            else
            {
                if ((lev==levmax) || (cabs(newpoint-oldpoint)<ep))
                {
                    if (x>1.5) //Limit set off screen
                    {
                        fl=1;
                        break;
                    }
                    plt=1;
                }
            }
            if (plt==1)
            {
                fl=1;
                z=newpoint-oldpoint;
                if (fabs(cimag(z))>fabs(creal(z)))
                {
                    stps=1;
                    x=ceil(fabs(scal*cimag(z)*siz2));
                    if (x>1) stps=x;
                }
                else
                {
                    stps=1;
                    x=ceil(fabs(scal*creal(z)*siz2));
                    if (x>1)  stps=x;
                }
                for (j=0;j<=stps;j++)
                {
                    z=scal*(newpoint*j/stps+oldpoint*(stps-j)/stps);
                    if ((fabs(creal(z))<1.0) && (fabs(cimag(z))<1.0))
                    {
                        if (iterate==0)
                            M[(int)(siz2-floor(sizs*creal(z)))][(int)(siz2-floor(sizs*cimag(z)))]=241;
                        if (iterate>0)
                        {
                            if (tags[iterate-1]==0) co=233; //blue
                            if (tags[iterate-1]==1) co=208; //red
                            if (tags[iterate-1]==2) co=165; //cyan
                            if (tags[iterate-1]==3) co=186; //magenta
                            M[(int)(siz2-floor(sizs*creal(z)))][(int)(siz2-floor(sizs*cimag(z)))]=co;
                        }
                        if (iterate==-1)
                        {
                            if (tags[lev]==0) co=233; //blue
                            if (tags[lev]==1) co=208; //red
                            if (tags[lev]==2) co=165; //cyan
                            if (tags[lev]==3) co=186; //magenta
                            M[(int)(siz2-floor(sizs*creal(z)))][(int)(siz2-floor(sizs*cimag(z)))]=co;
                        }
                        
                    }
                }
            }
            
            else
                fl=0;
            if (fl==1)
            {
            }
            else
            {
                lev=lev+1; //go forward
                tags[lev]=(tags[lev-1]+1)%4;
                if (lev==0)
                {
                    for (i=0;i<2;i++) for (j=0;j<2;j++)
                        word[i][j][lev]=gens[i][j][tags[lev]];
                    if (spec==2) state[lev]=tags[lev];
                }
                else
                {
                    for (i=0;i<2;i++) for (j=0;j<2;j++)
                    {
                        h1[i][j]=word[i][j][lev-1];
                        h2[i][j]=gens[i][j][tags[lev]];
                    }
                    mult(h1,h2,h3);
                    for (i=0;i<2;i++) for (j=0;j<2;j++)
                        word[i][j][lev]=h3[i][j];
                    if (spec==2)
                    {
                        state[lev]=FSA[state[lev-1]][tags[lev]];
                        if (state[lev]==-1) fl=1;
                    }
                }
            }
        }
        fl=1;
        while (fl==1)
        {
            lev=lev-1; //go backward
            if ((lev==-1) || ((tags[lev+1]+3)%4 != ((tags[lev]+2)%4))) //available turn
            {
                fl=0;
            }
        }
        if ((lev==-1)&&(tags[0]==1))
        {
           printf("%d %d %d %d\n", tags[0]+1,tags[1]+1,tags[2]+1,tags[3]+1);
           time (&end);
           dif=difftime (end,start);
           printf("Elasped time is %.0lf seconds.\n", dif );
           break; //finished!!
        }
        
        lev=lev+1;
        if (tags[lev]>0)
            tags[lev]=(tags[lev]-1)%4; //turnandgoforward
        else
            tags[lev]=(tags[lev]+3)%4;
        
        if (lev<3) //progress clock for long iterations
        {
            time (&end);
            dif=difftime (end,start);
            printf("%d %d %d %d ", tags[0]+1,tags[1]+1,tags[2]+1,tags[3]+1);
            printf("%.1lf sec\n", dif );
        }
        if (lev==0)
        {
            for (i=0;i<2;i++) for (j=0;j<2;j++)
            {
                word[i][j][lev]=gens[i][j][tags[lev]];
                //state[0]=tags[0];
            }
        }
        else
        {
            for (i=0;i<2;i++) for (j=0;j<2;j++)
            {
                h1[i][j]=word[i][j][lev-1];
                h2[i][j]=gens[i][j][tags[lev]];
            }
            mult(h1,h2,h3);
            for (i=0;i<2;i++) for (j=0;j<2;j++)
                word[i][j][lev]=h3[i][j];
            if (spec==2)
            {
                state[lev]=FSA[state[lev-1]][tags[lev]];
                lev=lev+1-1;
                // if (state[lev]==-1 fl=1;
            }
        }
        
    }
    fprintf(mf,"%s\n","P3");
    fprintf(mf,"%d %d\n",siz,siz);
    fprintf(mf,"%d\n",255);
    for(i=0;i<siz;i++)
        for(j=0;j<siz;j++)
        {
            switch (M[siz-1-j][i])
            {
                case 241:
                {
                    rd=0;
                    gr=0;
                    bl=0;
                    break;
                }
                case 233:
                {
                    rd=0;
                    gr=0;
                    bl=204;
                    break;
                }
                case 208:
                {
                    rd=204;
                    gr=0;
                    bl=0;
                    break;
                }
                case 165:
                {
                    rd=0;
                    gr=153;
                    bl=153;
                    break;
                }
                case 186:
                {
                    rd=153;
                    gr=0;
                    bl=153;
                    break;
                }
                default:
                {
                    rd=255;
                    gr=255;
                    bl=255;
                }
            }
            fprintf(mf,"%d %d %d ",rd,gr,bl);
        }
    fclose(mf);
    return(0);
}


double _Complex fixp(arry T)
{
    double _Complex z,tr,tr2,a,b,c,d;
    double _Complex k;
    a=T[0][0];
    b=T[0][1];
    c=T[1][0];
    d=T[1][1];
    tr=a+d;
    tr2=a*a+2*b*c+d*d;
    k=((tr+csqrt(tr2-4))/2)*((tr+csqrt(tr2-4))/2);
    if (cabs(k)>1)
        z=(-(d-a)+csqrt((d-a)*(d-a)+4*b*c))/(2*c);
    else
        z=(-(d-a)-csqrt((d-a)*(d-a)+4*b*c))/(2*c);
    return(z);
}


void mult(arry a,arry b,arry c)
{
    int i,j;
    for (i=0;i<2;i++)
        for (j=0;j<2;j++)
            c[i][j]=a[i][0]*b[0][j]+a[i][1]*b[1][j];
}

void inv(arry a, arry c)
{
    double _Complex d;
    d=a[0][0]*a[1][1]-a[0][1]*a[1][0];
    c[0][0]=a[1][1]/d;
    c[0][1]=-a[0][1]/d;
    c[1][0]=-a[1][0]/d;
    c[1][1]=a[0][0]/d;
}