// To run this file from terminal put it in a folder, cd to the folder and input
// gcc -o depthf depthfirstFSA.c -lm && ./depthf
// To run successive compiled versions type ./depthf
// To generate a compact png version of the .ppm image output file,
// first make a copy of the .ppm file and call it "test.ppm"
// Then use the following command in Terminal
// sips -s format png test.ppm --out test.png

#include <stdio.h>
#include <math.h>
#include <complex.h>
#include <time.h>

typedef double _Complex arry[2][2];

void mult(arry a,arry b, arry c);
void inv(arry a, arry c);
void asgn(arry a, arry c);
double _Complex fixp(arry T);

time_t start,end;

int main(void)
{
    FILE *mf;
    const char base[] = "DF";
    char filename [100];
    double _Complex tcomm = -2, offs=0;
    int spec=2, altroot=1, cnt=0; //spec=2 non-free group with FSA, altroot sets which square root for tab, cnt dot count
    double ep=0.0002,scal=1.5;
    int neck=1,iterate=0; //to aid drawing large whorls with narrow necks using midpoint
    int levmax=2000,siz=1100,rangeopt=0; //range opt sets escape options on range overflow
    // 1 gives escape 2 gives branch break. For doubly degenerate examples rangeopt=2 for time efficiency
     int anti=0; //if 1 choose symmetric anti-inverse
    // choose an example and then set the parameters according to the comments on the right
    //double _Complex ta=1.94730021-0.01952422i,tb=2; tcomm=-1; // 1/13 spec=3; altroot=0; scal=1; //activate only FSA[31]
   // double _Complex ta=1.94613967-0.01752014i ,tb=2; tcomm=-1.80193774; spec=7; altroot=0; scal=1; // 1/13 activate only FSA[79]
    //double _Complex ta=1.72435585-1.10404668i,tb=2; tcomm=-1; // spec=2; altroot=0; scal=1; // 144/233  activate only FSA[31]
    //double _Complex ta=1.63906337-1.25762377i,tb=2; tcomm=-1.80193774;   spec=2; altroot=0; scal=1; // 144/233activate only FSA[79]
    //double _Complex ta=1,tb=2; tcomm=-1; spec=2; altroot=0; scal=1.5; // a^3=I abAB^3=I activate only FSA[54]
   // double _Complex ta=2,tb=-(1+sqrt(5))/2; tcomm=-1; spec=2; altroot=0; scal=0.4; // b^5=I abAB^3=I activate only FSA[49]
   // double _Complex ta=1.93-0.05i,tb=-(1+sqrt(5))/2; tcomm=-1; spec=2; altroot=0; scal=0.4;  //b^5=I  abAB^3=I  activate only FSA[49] 
    //double _Complex ta=1.95+0.05i,tb=1; tcomm=-1.80193774; spec=2; altroot=1; scal=1.5;  //b^3=I  abAB^7=I   activate only FSA[116]
   // double _Complex ta=2,tb=2; tcomm=1; spec=2; altroot=0; ep=0.002; scal=0.8; offs=-1;//activate only FSA[19] 
 
   double _Complex ta=1.95+0.05i, tb=1.2495-1.7368i; tcomm=-1.24697960; levmax=22; spec=2; ep=0.003, altroot=0; scal=0.05; offs=-0.1+0.1i;//abAB^7=I   activate only FSA[79]

    //double _Complex ta=1.6088+0.6664i,tb=-1.6088+0.6664i; tcomm=0; levmax=22; spec=2; ep=0.005, altroot=1; scal=0.26; offs=0.4-0.1i; //tcomm=0  activate only FSA[19]  
    //double _Complex ta=1.95+0.05i, tb=1.2495-1.7368i; tcomm=-1.24697960; levmax=30; spec=2; ep=0.005, altroot=1; scal=0.05; offs=0;//abAB^7=I   activate only FSA[79]
    //double _Complex ta=1+1i, tb=1-1i; tcomm=1; levmax=20; spec=2; ep=0.005, altroot=0; scal=0.05; offs=0;//  activate only FSA[19]    
 //double _Complex ta=1.93+0.05i, tb=1; tcomm=-1.24697960; levmax=20; spec=2; ep=0.0005, altroot=1; scal=1.4; offs=-1.6i; anti=1; //abAB^7=I b^3=I  activate only FSA[116]
 //double _Complex ta=1.93+0.05i, tb=1; tcomm=-1.24697960; levmax=24; spec=2; ep=0.0005, altroot=1; scal=1.4; offs=-1.6;//abAB^7=I b^3=I  activate only FSA[116]
//double _Complex ta=1.93+0.05i, tb=1; tcomm=-1.24697960; levmax=20; spec=2; ep=0.0005, altroot=0; scal=0.4; offs=-0.4;//abAB^7=I b^3=I  activate only FSA[116]
//double _Complex ta=1i, tb=csqrt(2); tcomm=2+2.61803399; levmax=20; spec=2; ep=0.0003, altroot=0; scal=0.4; offs=0;//abAB^5=I b^4=I  activate only FSA[83]    
    // double _Complex ta=1.91+0.05i,tb=1.89-0.1i; tcomm=-1; spec=2; altroot=1; ep=0.0001; scal=0.32; offs=-0.7;//activate only FSA[19] 
    //double _Complex ta=1.905+0.05i,tb=1.905-0.05i; tcomm=-1; spec=2; altroot=1; ep=0.0001; scal=0.27; offs=-0.55;//activate only FSA[19] 
    

    int symm=1; //uses grandma if symm=0 and symmetric inverse if symm=1;
    int stpsz,lines=0;
    
    double _Complex tab;    
    int M[siz][siz],siz2,sizs;
    double _Complex gens[2][2][4];
    double _Complex word[3][3][levmax+1];
    int tags[levmax+1];
    int state[levmax+1];
  
    double dif;
    
    //int FSA[19][4]={{1,2,3,4},{1,5,-1,4},{7,2,8,-1},{-1,2,3,6},{9,-1,10,4},{7,2,11,-1},{12,-1,10,4},{1,5,-1,13},{-1,2,3,14},{1,15,-1,4},{-1,16,3,4},{-1,2,3,17},{1,18,-1,4},{9,-1,-1,4},{-1,-1,10,4},{7,2,-1,-1},{-1,2,8,-1},{-1,-1,10,4},{7,2,-1,-1}};  // tcomm=0,1

//int FSA[31][4]={{1,2,3,4},{1,5,-1,4},{6,2,7,-1},{-1,8,3,4},{9,-1,10,4},{6,2,11,-1},{1,5,-1,12},{-1,8,3,13},{14,2,7,-1},{1,15,-1,4},{-1,16,3,4},{-1,8,3,17},{9,-1,18,4},{19,-1,10,4},{1,5,-1,20},{6,2,21,-1},{22,2,7,-1},{23,-1,10,4},{-1,24,3,4},{1,25,-1,4},{9,-1,26,4},{-1,8,3,27},{1,5,-1,28},{1,29,-1,4},{-1,2,7,-1},{6,2,30,-1},{-1,-1,3,4},{-1,-1,10,4},{9,-1,-1,4},{6,5,-1,-1},{-1,8,7,-1}};  //tcomm=-1

int FSA[79][4]={{1,2,3,4},{1,5,-1,4},{6,2,7,-1},{-1,8,3,4},{9,-1,10,4},{6,2,11,-1},{1,5,-1,12},{-1,8,3,13},{14,2,7,-1},{1,15,-1,4},{-1,16,3,4},{-1,8,3,17},{9,-1,18,4},{19,-1,10,4},{1,5,-1,20},{6,2,21,-1},{22,2,7,-1},{23,-1,10,4},{-1,24,3,4},{1,25,-1,4},{9,-1,26,4},{-1,8,3,27},{1,5,-1,28},{1,29,-1,4},{30,2,7,-1},{6,2,31,-1},{-1,32,3,4},{33,-1,10,4},{9,-1,34,4},{6,2,35,-1},{1,5,-1,36},{-1,8,3,37},{38,2,7,-1},{1,39,-1,4},{-1,40,3,4},{-1,8,3,41},{9,-1,42,4},{43,-1,10,4},{1,5,-1,44},{6,2,45,-1},{46,2,7,-1},{47,-1,10,4},{-1,48,3,4},{1,49,-1,4},{9,-1,50,4},{-1,8,3,51},{1,5,-1,52},{1,53,-1,4},{54,2,7,-1},{6,2,55,-1},{-1,56,3,4},{57,-1,10,4},{9,-1,58,4},{6,2,59,-1},{1,5,-1,60},{-1,8,3,61},{62,2,7,-1},{1,63,-1,4},{-1,64,3,4},{-1,8,3,65},{9,-1,66,4},{67,-1,10,4},{1,5,-1,68},{6,2,69,-1},{70,2,7,-1},{71,-1,10,4},{-1,72,3,4},{1,73,-1,4},{9,-1,74,4},{-1,8,3,75},{1,5,-1,76},{1,77,-1,4},{-1,2,7,-1},{6,2,78,-1},{-1,-1,3,4},{-1,-1,10,4},{9,-1,-1,4},{6,5,-1,-1},{-1,8,7,-1}}; // tcomm=-1.80193774; abAB^7=I

//int FSA[54][4]={{1,2,3,4},{-1,5,-1,6},{7,2,8,-1},{-1,9,-1,10},{11,-1,12,4},{7,2,13,-1},{11,-1,14,4},{-1,5,-1,15},{-1,9,-1,16},{17,2,13,-1},{18,-1,14,4},{-1,19,-1,6},{-1,20,-1,10},{-1,9,-1,21},{-1,22,-1,10},{11,-1,23,4},{24,-1,14,4},{-1,5,-1,25},{-1,26,-1,6},{7,2,27,-1},{28,2,13,-1},{29,-1,14,4},{30,2,13,-1},{-1,31,-1,10},{-1,32,-1,6},{33,-1,34,4},{35,2,36,-1},{-1,9,-1,37},{-1,5,-1,38},{-1,39,-1,6},{-1,5,-1,40},{-1,2,41,-1},{42,2,43,-1},{-1,44,-1,6},{-1,-1,-1,10},{-1,5,-1,45},{-1,9,-1,46},{-1,-1,47,4},{48,-1,-1,4},{-1,5,-1,-1},{-1,-1,-1,6},{-1,9,-1,49},{-1,5,-1,50},{-1,9,-1,-1},{17,2,27,-1},{18,-1,23,4},{-1,-1,-1,51},{-1,52,-1,10},{-1,53,-1,10},{29,-1,23,4},{24,-1,23,4},{18,-1,12,4},{30,2,27,-1},{28,2,27,-1}}; //a^3=I abAB^3=I tcomm=-1 ta=1

//int FSA[49][4]={{1,2,3,4},{1,5,-1,4},{6,7,8,-1},{-1,9,3,4},{10,-1,11,12},{6,7,13,-1},{1,5,-1,14},{15,-1,8,-1},{-1,9,3,16},{17,7,8,-1},{1,18,-1,4},{-1,19,3,4},{20,-1,21,-1},{-1,9,3,22},{10,-1,23,12},{1,24,-1,14},{25,-1,11,12},{1,5,-1,26},{6,7,27,-1},{28,7,8,-1},{1,18,-1,29},{-1,19,3,16},{30,-1,11,12},{-1,31,3,4},{6,7,32,-1},{1,33,-1,4},{10,-1,34,12},{-1,9,3,35},{1,5,-1,36},{10,-1,37,12},{1,38,-1,4},{-1,7,8,-1},{-1,9,3,39},{6,7,40,-1},{-1,-1,3,4},{-1,-1,11,12},{10,-1,-1,12},{-1,41,3,4},{6,42,-1,-1},{43,-1,11,12},{-1,44,8,-1},{45,7,8,-1},{15,-1,13,-1},{46,-1,-1,4},{47,-1,8,-1},{48,42,-1,-1},{1,24,-1,4},{1,24,-1,26},{1,5,-1,29}}; //b^5=I  abAB^3=I tcomm=-1 tb=-(1+sqrt(5))/2

//int FSA[116][4]={{1,2,3,4},{1,5,-1,4},{6,-1,7,-1},{-1,8,3,4},{9,-1,10,-1},{6,-1,11,-1},{1,12,-1,13},{-1,8,3,14},{15,-1,7,-1},{1,16,-1,17},{-1,18,3,14},{-1,8,3,19},{6,-1,20,-1},{9,-1,21,-1},{22,-1,10,-1},{1,12,-1,23},{6,-1,24,-1},{9,-1,25,-1},{26,-1,7,-1},{27,-1,10,-1},{-1,8,3,28},{-1,29,3,14},{1,30,-1,17},{9,-1,31,-1},{-1,8,3,32},{-1,33,3,14},{1,12,-1,34},{1,35,-1,17},{36,-1,10,-1},{37,-1,7,-1},{6,-1,38,-1},{-1,39,3,14},{40,-1,10,-1},{41,-1,7,-1},{9,-1,42,-1},{6,-1,43,-1},{1,44,-1,17},{1,12,-1,45},{-1,8,3,46},{47,-1,7,-1},{1,48,-1,17},{1,12,-1,49},{-1,50,3,14},{-1,8,3,51},{6,-1,52,-1},{9,-1,53,-1},{54,-1,10,-1},{1,12,-1,55},{6,-1,56,-1},{9,-1,57,-1},{58,-1,7,-1},{59,-1,10,-1},{-1,8,3,60},{-1,61,3,14},{1,62,-1,17},{9,-1,63,-1},{-1,8,3,64},{-1,65,3,14},{1,12,-1,66},{1,67,-1,17},{68,-1,10,-1},{69,-1,7,-1},{6,-1,70,-1},{-1,71,3,14},{72,-1,10,-1},{73,-1,7,-1},{9,-1,74,-1},{6,-1,75,-1},{1,76,-1,17},{1,12,-1,77},{-1,8,3,78},{79,-1,7,-1},{1,80,-1,17},{1,12,-1,81},{-1,82,3,14},{-1,8,3,83},{6,-1,84,-1},{9,-1,85,-1},{86,-1,10,-1},{1,12,-1,87},{6,-1,88,-1},{9,-1,89,-1},{90,-1,7,-1},{91,-1,10,-1},{-1,8,3,92},{-1,93,3,94},{1,95,-1,17},{9,-1,96,-1},{-1,97,3,98},{-1,99,3,14},{1,100,-1,101},{1,102,-1,103},{104,-1,10,-1},{-1,-1,7,-1},{105,-1,10,-1},{6,-1,93,-1},{-1,-1,3,106},{107,-1,7,-1},{-1,-1,10,-1},{108,-1,7,-1},{6,-1,109,-1},{9,-1,-1,-1},{6,-1,-1,-1},{9,-1,110,-1},{111,-1,-1,112},{1,30,-1,13},{113,-1,10,-1},{1,16,-1,23},{114,-1,-1,-1},{-1,18,3,28},{-1,33,3,19},{1,12,-1,4},{9,-1,115,-1},{1,30,-1,23},{1,12,-1,17},{-1,33,3,28}};  //b^3=I  abAB^7=I tb=1

//int FSA[83][4]={{1,2,3,4},{1,5,-1,4},{6,7,8,-1},{-1,9,3,4},{10,-1,11,-1},{6,7,12,-1},{1,5,-1,13},{14,-1,8,-1},{-1,9,3,15},{16,7,8,-1},{1,17,-1,18},{-1,19,3,15},{-1,9,3,20},{10,-1,21,-1},{1,22,-1,13},{23,-1,11,-1},{1,5,-1,24},{6,7,25,-1},{10,-1,26,-1},{27,7,8,-1},{28,-1,11,-1},{-1,29,3,15},{6,7,30,-1},{1,31,-1,18},{10,-1,32,-1},{-1,9,3,33},{-1,34,3,15},{1,5,-1,35},{1,36,-1,18},{37,7,8,-1},{-1,9,3,38},{6,7,39,-1},{-1,40,3,15},{41,-1,11,-1},{42,7,8,-1},{10,-1,43,-1},{6,7,44,-1},{1,5,-1,45},{46,-1,11,-1},{-1,9,3,47},{48,7,8,-1},{1,49,-1,18},{1,5,-1,50},{-1,51,3,15},{-1,9,3,52},{10,-1,53,-1},{1,54,-1,18},{55,-1,11,-1},{1,5,-1,56},{6,7,57,-1},{10,-1,58,-1},{59,7,8,-1},{60,-1,11,-1},{-1,61,3,15},{6,7,62,-1},{1,63,-1,18},{10,-1,64,-1},{-1,65,3,66},{-1,67,3,15},{1,68,-1,69},{1,70,-1,18},{-1,7,8,-1},{-1,9,3,71},{6,7,72,-1},{-1,-1,3,15},{16,73,8,-1},{-1,-1,11,-1},{74,7,8,-1},{6,75,12,-1},{10,-1,-1,-1},{6,76,-1,-1},{77,-1,11,-1},{-1,78,8,-1},{79,-1,8,-1},{80,76,-1,-1},{14,-1,11,-1},{14,-1,12,-1},{81,-1,-1,18},{82,-1,8,-1},{1,17,-1,13},{1,5,-1,18},{1,22,-1,4},{1,22,-1,24}};  //b^4=I abAB^5=I


//int FSA[638][4]={{1,2,3,4},{5,-1,6,7},{-1,2,8,9},{10,11,-1,-1},{12,13,-1,-1},{5,-1,14,7},{10,15,-1,-1},{12,16,-1,-1},{17,11,-1,-1},{18,13,-1,-1},{5,-1,19,20},{-1,2,21,22},{5,-1,23,24},{-1,2,25,26},{10,27,-1,-1},{-1,2,21,28},{-1,2,29,26},{5,-1,19,30},{5,-1,31,24},{10,32,-1,-1},{12,33,-1,-1},{34,11,-1,-1},{35,13,-1,-1},{10,36,-1,-1},{12,37,-1,-1},{38,11,-1,-1},{39,13,-1,-1},{-1,2,21,40},{41,13,-1,-1},{42,11,-1,-1},{12,43,-1,-1},{10,44,-1,-1},{-1,2,21,45},{-1,2,46,26},{5,-1,19,47},{5,-1,48,24},{-1,2,21,49},{-1,2,50,26},{5,-1,19,51},{5,-1,52,24},{53,13,-1,-1},{5,-1,54,55},{5,-1,56,57},{-1,2,58,59},{-1,2,60,61},{62,13,-1,-1},{63,11,-1,-1},{12,64,-1,-1},{10,65,-1,-1},{66,13,-1,-1},{67,11,-1,-1},{12,68,-1,-1},{10,69,-1,-1},{5,-1,70,55},{10,71,-1,-1},{12,72,-1,-1},{10,73,-1,-1},{12,74,-1,-1},{75,11,-1,-1},{76,13,-1,-1},{77,11,-1,-1},{78,13,-1,-1},{5,-1,79,80},{81,-1,82,83},{-1,2,84,85},{-1,86,87,88},{89,-1,90,91},{5,-1,92,93},{-1,94,95,96},{-1,2,97,98},{10,99,-1,-1},{-1,100,-1,101},{-1,2,50,102},{-1,2,103,45},{-1,104,-1,-1},{105,-1,-1,-1},{5,-1,52,106},{5,-1,107,47},{108,-1,-1,-1},{10,-1,-1,-1},{12,109,-1,-1},{5,-1,14,110},{111,112,-1,-1},{113,-1,-1,-1},{-1,11,-1,-1},{114,13,-1,-1},{-1,2,8,115},{116,117,-1,-1},{-1,118,-1,-1},{5,-1,119,7},{120,-1,-1,-1},{121,122,-1,-1},{10,123,-1,-1},{12,-1,-1,-1},{-1,2,124,9},{-1,125,-1,-1},{126,127,-1,-1},{128,11,-1,-1},{-1,13,-1,-1},{-1,100,-1,-1},{-1,2,21,102},{129,-1,-1,-1},{130,13,-1,-1},{131,11,-1,-1},{-1,2,103,132},{5,-1,19,106},{12,133,-1,-1},{10,134,-1,-1},{5,-1,107,135},{-1,2,50,136},{12,137,-1,-1},{5,-1,138,20},{-1,2,139,45},{140,-1,141,142},{5,-1,52,143},{144,13,-1,-1},{5,-1,145,47},{-1,2,146,22},{-1,147,148,149},{10,150,-1,-1},{140,-1,151,152},{5,-1,23,106},{-1,2,50,153},{-1,2,154,45},{155,11,-1,-1},{-1,147,156,157},{5,-1,52,158},{-1,2,25,102},{5,-1,159,47},{-1,-1,160,-1},{5,-1,161,24},{5,-1,19,162},{163,13,-1,-1},{-1,2,164,26},{-1,2,21,165},{12,166,-1,-1},{167,13,-1,-1},{-1,2,139,26},{10,168,-1,-1},{169,11,-1,-1},{140,-1,170,171},{172,173,-1,-1},{174,175,-1,-1},{12,176,-1,-1},{5,-1,145,24},{10,177,-1,-1},{178,11,-1,-1},{-1,147,179,180},{181,182,-1,-1},{183,184,-1,-1},{-1,2,21,153},{172,185,-1,-1},{174,186,-1,-1},{187,13,-1,-1},{188,11,-1,-1},{5,-1,19,158},{189,182,-1,-1},{190,184,-1,-1},{12,191,-1,-1},{10,192,-1,-1},{-1,193,-1,-1},{10,194,-1,-1},{12,195,-1,-1},{5,-1,196,24},{197,11,-1,-1},{198,13,-1,-1},{-1,2,199,26},{5,-1,200,24},{-1,2,21,201},{5,-1,202,93},{172,203,-1,-1},{174,204,-1,-1},{140,-1,205,206},{-1,147,207,208},{140,-1,209,142},{-1,147,210,149},{-1,2,211,26},{-1,2,212,98},{5,-1,19,213},{214,182,-1,-1},{215,184,-1,-1},{140,-1,205,216},{-1,147,207,217},{140,-1,218,142},{-1,147,219,149},{-1,147,207,220},{-1,147,221,149},{5,-1,79,222},{5,-1,223,224},{140,-1,205,225},{140,-1,226,142},{-1,2,84,227},{-1,2,228,229},{-1,-1,-1,230},{-1,2,231,98},{-1,2,84,232},{10,233,-1,-1},{5,-1,234,93},{5,-1,79,235},{236,11,-1,-1},{10,237,-1,-1},{238,13,-1,-1},{10,239,-1,-1},{-1,147,207,240},{-1,147,241,149},{172,242,-1,-1},{174,243,-1,-1},{244,182,-1,-1},{245,184,-1,-1},{172,246,-1,-1},{247,182,-1,-1},{248,11,-1,-1},{249,11,-1,-1},{12,250,-1,-1},{140,-1,205,251},{140,-1,252,142},{174,253,-1,-1},{254,184,-1,-1},{172,255,-1,-1},{256,182,-1,-1},{257,184,-1,-1},{258,182,-1,-1},{12,259,-1,-1},{10,260,-1,-1},{12,261,-1,-1},{174,262,-1,-1},{172,263,-1,-1},{264,13,-1,-1},{265,11,-1,-1},{266,13,-1,-1},{267,-1,-1,-1},{268,11,-1,-1},{269,13,-1,-1},{-1,2,270,98},{10,271,-1,-1},{12,272,-1,-1},{5,-1,273,93},{-1,274,275,-1},{5,-1,79,276},{-1,2,25,45},{277,184,-1,-1},{278,182,-1,-1},{-1,147,207,279},{-1,147,280,149},{140,-1,205,281},{140,-1,282,283},{-1,147,207,284},{140,-1,285,286},{287,-1,288,-1},{5,-1,23,47},{-1,2,84,289},{174,290,-1,-1},{172,291,-1,-1},{-1,147,292,293},{140,-1,294,142},{-1,147,295,296},{140,-1,205,297},{140,-1,282,298},{140,-1,299,286},{-1,2,50,22},{-1,2,300,45},{-1,301,-1,-1},{-1,147,292,302},{-1,147,303,296},{5,-1,52,20},{5,-1,304,47},{305,-1,-1,-1},{174,-1,306,-1},{5,-1,31,47},{5,-1,52,30},{307,11,-1,-1},{-1,2,29,45},{-1,2,50,40},{10,308,-1,-1},{-1,2,8,132},{309,310,-1,-1},{12,311,-1,-1},{140,-1,282,312},{140,-1,313,286},{314,184,-1,-1},{315,182,-1,-1},{174,316,-1,-1},{172,-1,-1,-1},{174,317,-1,-1},{318,184,-1,-1},{172,319,-1,-1},{174,-1,-1,-1},{5,-1,14,135},{320,321,-1,-1},{322,13,-1,-1},{-1,147,292,323},{-1,147,324,296},{-1,182,-1,-1},{325,184,-1,-1},{172,326,-1,-1},{327,182,-1,-1},{-1,184,-1,-1},{174,328,-1,-1},{174,329,-1,-1},{172,330,-1,-1},{331,11,-1,-1},{-1,2,300,332},{333,184,-1,-1},{334,182,-1,-1},{10,335,-1,-1},{5,-1,304,336},{140,147,337,-1},{5,-1,338,47},{-1,2,339,45},{5,-1,196,47},{-1,2,103,22},{-1,2,50,340},{174,341,-1,-1},{172,342,-1,-1},{140,-1,282,343},{344,-1,345,-1},{-1,147,292,346},{-1,147,210,240},{347,-1,-1,348},{-1,147,349,279},{5,-1,107,20},{-1,2,199,45},{5,-1,52,350},{351,184,-1,-1},{352,182,-1,-1},{140,-1,218,251},{-1,353,354,-1},{140,-1,355,281},{-1,356,-1,357},{-1,147,210,217},{-1,147,241,279},{5,-1,19,358},{359,13,-1,-1},{140,-1,218,206},{140,-1,252,281},{-1,2,21,360},{12,361,-1,-1},{172,182,-1,-1},{10,362,-1,-1},{363,11,-1,-1},{364,13,-1,-1},{-1,147,210,157},{-1,147,148,279},{174,365,-1,-1},{140,-1,170,366},{367,368,-1,-1},{369,184,-1,-1},{140,-1,151,171},{174,370,-1,-1},{371,182,-1,-1},{12,372,-1,-1},{140,-1,218,152},{140,-1,141,281},{-1,147,179,373},{374,375,-1,-1},{172,376,-1,-1},{-1,147,156,180},{377,184,-1,-1},{12,378,-1,-1},{5,-1,379,24},{380,13,-1,-1},{-1,2,381,26},{-1,2,21,382},{5,-1,19,383},{5,-1,384,55},{-1,147,210,385},{174,386,-1,-1},{140,-1,387,206},{-1,147,388,279},{140,-1,218,389},{-1,147,210,220},{140,-1,390,391},{-1,2,392,59},{393,184,-1,-1},{140,-1,394,281},{-1,147,395,217},{-1,147,396,397},{140,-1,218,225},{-1,2,84,398},{10,399,-1,-1},{5,-1,79,400},{401,11,-1,-1},{402,13,-1,-1},{12,403,-1,-1},{10,404,-1,-1},{405,184,-1,-1},{-1,147,388,149},{172,406,-1,-1},{407,182,-1,-1},{174,408,-1,-1},{172,409,-1,-1},{174,410,-1,-1},{411,11,-1,-1},{140,-1,394,142},{172,412,-1,-1},{413,182,-1,-1},{414,182,-1,-1},{415,184,-1,-1},{416,13,-1,-1},{-1,2,417,418},{12,419,-1,-1},{5,-1,420,421},{422,-1,-1,423},{-1,424,-1,425},{-1,426,427,-1},{140,-1,428,142},{-1,147,207,429},{140,-1,430,286},{-1,147,431,149},{-1,147,395,279},{-1,432,-1,-1},{433,-1,434,-1},{-1,147,435,296},{140,-1,205,436},{140,-1,387,281},{437,-1,-1,-1},{5,-1,52,438},{439,11,-1,-1},{440,13,-1,-1},{-1,2,50,441},{10,442,-1,-1},{12,443,-1,-1},{5,-1,19,7},{121,444,-1,-1},{-1,2,21,9},{445,127,-1,-1},{-1,2,124,446},{447,448,-1,-1},{172,449,-1,-1},{450,184,-1,-1},{172,451,-1,-1},{452,182,-1,-1},{-1,147,395,373},{5,-1,119,453},{454,455,-1,-1},{456,182,-1,-1},{174,457,-1,-1},{140,-1,387,366},{12,458,-1,-1},{5,-1,459,47},{460,-1,-1,-1},{461,13,-1,-1},{-1,2,462,45},{-1,463,-1,-1},{-1,2,50,45},{5,-1,52,47},{464,13,-1,-1},{140,-1,465,225},{-1,147,466,157},{-1,467,468,-1},{140,-1,282,469},{-1,147,219,279},{470,-1,471,-1},{12,472,-1,-1},{140,-1,473,152},{-1,147,474,220},{140,-1,209,281},{-1,147,292,475},{-1,2,476,26},{10,477,-1,-1},{140,-1,473,478},{5,-1,479,24},{480,11,-1,-1},{-1,147,466,481},{5,-1,482,24},{172,483,-1,-1},{484,182,-1,-1},{-1,147,179,481},{485,486,-1,-1},{174,487,-1,-1},{140,-1,170,478},{488,489,-1,-1},{-1,2,490,26},{172,491,-1,-1},{492,182,-1,-1},{493,184,-1,-1},{494,11,-1,-1},{-1,2,21,495},{174,496,-1,-1},{10,497,-1,-1},{5,-1,19,498},{499,184,-1,-1},{10,500,-1,-1},{-1,147,501,296},{140,-1,205,502},{140,-1,465,281},{-1,147,466,217},{-1,147,210,503},{140,-1,473,206},{-1,147,474,279},{504,11,-1,-1},{-1,147,207,505},{140,-1,506,286},{140,-1,218,507},{508,-1,-1,-1},{509,13,-1,-1},{-1,147,474,149},{-1,510,-1,-1},{12,511,-1,-1},{140,-1,465,142},{-1,2,512,98},{513,182,-1,-1},{174,514,-1,-1},{515,184,-1,-1},{5,-1,516,93},{517,184,-1,-1},{172,518,-1,-1},{174,519,-1,-1},{5,-1,520,24},{521,-1,-1,522},{-1,2,523,26},{-1,524,-1,525},{526,11,-1,-1},{140,-1,527,281},{-1,356,-1,528},{140,-1,529,142},{10,530,-1,-1},{347,-1,-1,531},{-1,147,532,279},{-1,147,533,149},{10,534,-1,-1},{5,-1,520,7},{12,535,-1,-1},{536,11,-1,-1},{-1,2,523,9},{537,13,-1,-1},{5,-1,538,47},{172,539,-1,-1},{540,184,-1,-1},{172,541,-1,-1},{-1,2,542,45},{174,543,-1,-1},{544,182,-1,-1},{545,182,-1,-1},{-1,2,21,546},{-1,2,50,546},{5,-1,19,547},{5,-1,52,547},{10,548,-1,-1},{-1,147,207,549},{140,-1,218,550},{-1,551,-1,-1},{552,11,-1,-1},{-1,147,210,553},{140,-1,205,554},{555,-1,-1,-1},{556,13,-1,-1},{12,557,-1,-1},{-1,2,558,559},{560,184,-1,-1},{174,561,-1,-1},{-1,147,156,149},{5,-1,562,563},{564,184,-1,-1},{174,565,-1,-1},{140,-1,151,142},{5,-1,566,567},{-1,2,568,569},{570,11,-1,-1},{571,13,-1,-1},{572,-1,-1,573},{-1,147,574,575},{10,576,-1,-1},{12,577,-1,-1},{140,-1,578,579},{-1,580,-1,581},{10,582,-1,-1},{12,583,-1,-1},{584,11,-1,-1},{585,13,-1,-1},{5,-1,138,47},{586,-1,-1,587},{140,-1,205,171},{588,589,-1,-1},{584,182,-1,-1},{590,184,-1,-1},{-1,2,146,45},{-1,591,-1,592},{172,582,-1,-1},{174,593,-1,-1},{-1,147,207,180},{594,595,-1,-1},{-1,596,-1,-1},{-1,2,50,597},{598,-1,-1,-1},{5,-1,52,599},{5,-1,138,110},{600,589,-1,-1},{140,-1,209,601},{-1,147,210,279},{140,-1,218,601},{-1,2,146,115},{594,602,-1,-1},{-1,147,210,603},{140,-1,218,281},{-1,147,219,603},{-1,147,207,603},{604,13,-1,-1},{140,-1,205,601},{12,605,-1,-1},{140,-1,141,601},{174,606,-1,-1},{-1,147,148,603},{607,184,-1,-1},{5,-1,608,24},{-1,2,609,26},{-1,147,610,149},{140,-1,611,142},{10,612,-1,-1},{613,11,-1,-1},{614,182,-1,-1},{172,615,-1,-1},{-1,616,617,-1},{618,-1,619,-1},{344,-1,620,-1},{-1,353,621,-1},{-1,2,8,332},{622,623,-1,-1},{5,-1,14,336},{624,625,-1,-1},{367,626,-1,-1},{627,375,-1,-1},{5,-1,379,47},{-1,2,300,22},{5,-1,304,20},{-1,2,381,45},{-1,147,628,279},{140,-1,629,281},{630,182,-1,-1},{172,631,-1,-1},{140,-1,632,633},{-1,147,634,635},{172,636,-1,-1},{174,443,-1,-1},{637,182,-1,-1},{440,184,-1,-1},{-1,147,466,279},{140,-1,473,281}};  //b^3=I  abAB^7=I full
    
    double _Complex fix[3][4];
    int i,j,co,rd,gr,bl;
    double _Complex p,q,z0,z,Q,tC,R,zo;
    arry a,b,ab,A,B,h;
    arry *iv, *mu,h1,h2,h3;
    int stfl,lev,fl,plt,stps;
    double _Complex oldpoint,stpt,newpoint,midpoint;
    double x;
    
    sprintf(filename, "%s %e %e %e %e %d %e %e.ppm",base,creal(ta),cimag(ta),creal(tb),cimag(tb),levmax,ep,scal);
    mf=fopen(filename,"w");
    time (&start);
    ep=ep/scal;
    for (i=0;i<siz;i++)
        for (j=0;j<siz;j++)
            M[i][j]=255;
    siz2=siz/2;
    sizs=(int)siz2*10.0/11;
    //grandma's recipe
    if (spec==0)
    {
        p=-ta*tb;
        q=ta*ta+tb*tb;
        if (altroot==0)
            tab=(-p-csqrt(p*p-4*q))/2;
        else
            tab=(-p+csqrt(p*p-4*q))/2;
        z0=((tab-2)*tb)/(tb*tab-2*ta+2*I*tab);
        ab[0][0]=tab/2;
        ab[0][1]=(tab-2)/(2*z0);
        ab[1][0]=(tab+2)*z0/2;
        ab[1][1]=tab/2;
        b[0][0]=(tb-2*I)/2;
        b[0][1]=tb/2;
        b[1][0]=tb/2;
        b[1][1]=(tb+2*I)/2;
    }
    else
    {
        p=-ta*tb;
	q=ta*ta+tb*tb-tcomm-2;
        if (altroot==0)
            tab=(-p-csqrt(p*p-4*q))/2;
        else
            tab=(-p+csqrt(p*p-4*q))/2;
        tC=ta*ta+tb*tb+tab*tab-ta*tb*tab-2; //Grandma's three parameter double alarm
        Q=csqrt(2-tC);
        if ((cabs(tC+I*Q*csqrt(tC+2))>=2+1e-15))
            R=csqrt(tC+2);
        else
            R=-csqrt(tC+2);
        zo=(tab-2)*(tb-R)/(tb*tab-2*ta+1i*Q*tab);
        a[0][0]=ta/2;
        a[0][1]=(ta*tab-2*tb+2*I*Q)/((2*tab+4)*zo);
        a[1][0]=(ta*tab-2*tb-2*I*Q)*zo/(2*tab-4);
        a[1][1]=ta/2;
        b[0][0]=(tb-I*Q)/2;
        b[0][1]=(tb*tab-2*ta-I*Q*tab)/((2*tab+4)*zo);
        b[1][0]=(tb*tab-2*ta+I*Q*tab)*zo/(2*tab-4);
        b[1][1]=(tb+I*Q)/2;
     if (symm==1)
        {
        inv(a,h1); //grandma to symmetric
	inv(b,h2);
	mult(h1,h2,a);
	asgn(h1,b);

	if(anti==0)
	{
	h1[0][0]=1; // symm inverse
	h1[0][1]=1;
	h1[1][0]=-1;
	h1[1][1]=1;
	}
	else
	{
	h1[0][0]=0.5*(1+1i); // symm inverse
	h1[0][1]=0.5*(-1-1i);
	h1[1][0]=0.5*(-1+1i);
	h1[1][1]=0.5*(-1+1i);
	}
	inv(h1,h2);
	mult(h1,a,h3); mult(h3,h2,a);
	mult(h1,b,h3); mult(h3,h2,b);}   
   	}
    inv(b,B);
    if (spec==0)
        mult(ab,B,a);
    inv(a,A);
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][0]=a[i][j];
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][1]=b[i][j];
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][2]=A[i][j];
    for (i=0;i<2;i++) for (j=0;j<2;j++) gens[i][j][3]=B[i][j];
    mult(b,a,h1); mult(A,h1,h2); mult(B,h2,h3); fix[0][0]=fixp(h3);
    mult(A,b,h1); mult(B,h1,h2); mult(a,h2,h3); fix[0][1]=fixp(h3);
    mult(B,A,h1); mult(a,h1,h2); mult(b,h2,h3); fix[0][2]=fixp(h3);
    mult(a,B,h1); mult(b,h1,h2); mult(A,h2,h3); fix[0][3]=fixp(h3);
    fix[1][0]=fixp(a); fix[1][1]=fixp(b); fix[1][2]=fixp(A); fix[1][3]=fixp(B);
    mult(B,a,h1); mult(A,h1,h2); mult(b,h2,h3); fix[2][0]=fixp(h3);
    mult(a,b,h1); mult(B,h1,h2); mult(A,h2,h3); fix[2][1]=fixp(h3);
    mult(b,A,h1); mult(a,h1,h2); mult(B,h2,h3); fix[2][2]=fixp(h3);
    mult(A,B,h1); mult(b,h1,h2); mult(a,h2,h3); fix[2][3]=fixp(h3);
    lev=0;
    tags[0]=0;
    state[0]=0;
    for (i=0;i<2;i++) for (j=0;j<2;j++) word[i][j][0]=gens[i][j][0];
    
    while (~((lev==-1)&&(tags[0]==1)))
    {
        fl=0;
        if (spec>=2)
            if (state[lev]==-1)
                fl=1;
        while (fl==0)
        {
            z=fix[0][tags[lev]];
            newpoint=(word[0][0][lev]*z+word[0][1][lev])/(word[1][0][lev]*z+word[1][1][lev]);
            z=fix[2][tags[lev]];
            oldpoint=(word[0][0][lev]*z+word[0][1][lev])/(word[1][0][lev]*z+word[1][1][lev]);
            if (neck>0)
            {
                z=fix[1][tags[lev]];
                midpoint=(word[0][0][lev]*z+word[0][1][lev])/(word[1][0][lev]*z+word[1][1][lev]);
            }
            plt=0;
            //x=scal*cabs(newpoint)+offs;
	    x=cabs(scal*newpoint+offs);
            if (rangeopt>0)
                if (x>10) //Chaotic instability
            {
                if (rangeopt==1)
                {
                    printf("exiting...\n");
                    return(0);
                }
                if (rangeopt==2)
                {
                    printf(".");
                    cnt=cnt+1;
                    if (cnt>200)
                    {
                        printf("\n");
                        cnt=0;
                    }
                    fl=1;
                    break;
                }
            }
            if (neck>0)
            {
                if ((lev==levmax) || ((cabs(newpoint-midpoint)<ep)&&(cabs(midpoint-oldpoint)<ep)))
                {
                    if (x>1.5) //Limit set off screen
                    {
                        fl=1;
                        break;
                    }
                    plt=1;
                }
            }
            else
            {
                if ((lev==levmax) || (cabs(newpoint-oldpoint)<ep))
                {
                    if (x>1.5) //Limit set off screen
                    {
                        fl=1;
                        break;
                    }
                    plt=1;
                }
            } 
	    if (lev==levmax)
	    {
		fl=1;
		break;
	    }
            if (plt==1)
            {
                fl=1;
                z=newpoint-oldpoint;
                if (fabs(cimag(z))>fabs(creal(z)))
                {
                    stps=1;
                    x=ceil(fabs(scal*cimag(z)*siz2));
                    if (x>1) stps=x;
                }
                else
                {
                    stps=1;
                    x=ceil(fabs(scal*creal(z)*siz2));
                    if (x>1)  stps=x;
                }
		if(lines==0)
			stpsz=stps;
                else
                        stpsz=1;
                for (j=0;j<=stps;j=j+stps)
                {
                    z=scal*(newpoint*j/stps+oldpoint*(stps-j)/stps)+offs;
                    if ((fabs(creal(z))<1.0) && (fabs(cimag(z))<1.0))
                    {
                        if (iterate==0)
                            M[(int)(siz2-floor(sizs*creal(z)))][(int)(siz2-floor(sizs*cimag(z)))]=241;
                        if (iterate>0)
                        {
                            if (tags[iterate-1]==0) co=233; //blue
                            if (tags[iterate-1]==1) co=208; //red
                            if (tags[iterate-1]==2) co=165; //cyan
                            if (tags[iterate-1]==3) co=186; //magenta
                            M[(int)(siz2-floor(sizs*creal(z)))][(int)(siz2-floor(sizs*cimag(z)))]=co;
                        }
                        if (iterate==-1)
                        {
                            if (tags[lev]==0) co=233; //blue
                            if (tags[lev]==1) co=208; //red
                            if (tags[lev]==2) co=165; //cyan
                            if (tags[lev]==3) co=186; //magenta
                            M[(int)(siz2-floor(sizs*creal(z)))][(int)(siz2-floor(sizs*cimag(z)))]=co;
                        }
                        
                    }
                }
            }
            
            else
                fl=0;
            if (fl==1)
            {
            }
            else
            {
                lev=lev+1; //go forward
                tags[lev]=(tags[lev-1]+1)%4;
                if (lev==0)
                {
                    for (i=0;i<2;i++) for (j=0;j<2;j++)
                        word[i][j][lev]=gens[i][j][tags[lev]];
                    if (spec>=2) state[lev]=tags[lev];
                }
                else
                {
                    for (i=0;i<2;i++) for (j=0;j<2;j++)
                    {
                        h1[i][j]=word[i][j][lev-1];
                        h2[i][j]=gens[i][j][tags[lev]];
                    }
                    mult(h1,h2,h3);
                    for (i=0;i<2;i++) for (j=0;j<2;j++)
                        word[i][j][lev]=h3[i][j];
                    if (spec>=2)
                    {
                        state[lev]=FSA[state[lev-1]][tags[lev]];
                        if (state[lev]==-1) fl=1;
                    }
                }
            }
        }
        fl=1;
        while (fl==1)
        {
            lev=lev-1; //go backward
            if ((lev==-1) || ((tags[lev+1]+3)%4 != ((tags[lev]+2)%4))) //available turn
            {
                fl=0;
            }
        }
        if ((lev==-1)&&(tags[0]==1))
        {
           printf("%d %d %d %d\n", tags[0]+1,tags[1]+1,tags[2]+1,tags[3]+1);
           time (&end);
           dif=difftime (end,start);
           printf("Elasped time is %.0lf seconds.\n", dif );
           break; //finished!!
        }
        
        lev=lev+1;
        if (tags[lev]>0)
            tags[lev]=(tags[lev]-1)%4; //turn and go forward
        else
            tags[lev]=(tags[lev]+3)%4;
        
        if (lev<3) //progress clock for long iterations
        {
            time (&end);
            dif=difftime (end,start);
            printf("%d %d %d %d ", tags[0]+1,tags[1]+1,tags[2]+1,tags[3]+1);
            printf("%.1lf sec\n", dif );
        }
        if (lev==0)
        {
            for (i=0;i<2;i++) for (j=0;j<2;j++)
            {
                word[i][j][lev]=gens[i][j][tags[lev]];
                //state[0]=tags[0];
            }
        }
        else
        {
            for (i=0;i<2;i++) for (j=0;j<2;j++)
            {
                h1[i][j]=word[i][j][lev-1];
                h2[i][j]=gens[i][j][tags[lev]];
            }
            mult(h1,h2,h3);
            for (i=0;i<2;i++) for (j=0;j<2;j++)
                word[i][j][lev]=h3[i][j];
            if (spec>=2)
            {
                state[lev]=FSA[state[lev-1]][tags[lev]];
                lev=lev+1-1;
                // if (state[lev]==-1 fl=1;
            }
        }
        
    }
    fprintf(mf,"%s\n","P3");
    fprintf(mf,"%d %d\n",siz,siz);
    fprintf(mf,"%d\n",255);
    for(i=0;i<siz;i++)
        for(j=0;j<siz;j++)
        {
            switch (M[siz-1-j][i])
            {
                case 241:
                {
                    rd=0;
                    gr=0;
                    bl=0;
                    break;
                }
                case 233:
                {
                    rd=0;
                    gr=0;
                    bl=204;
                    break;
                }
                case 208:
                {
                    rd=204;
                    gr=0;
                    bl=0;
                    break;
                }
                case 165:
                {
                    rd=0;
                    gr=153;
                    bl=153;
                    break;
                }
                case 186:
                {
                    rd=153;
                    gr=0;
                    bl=153;
                    break;
                }
                default:
                {
                    rd=255;
                    gr=255;
                    bl=255;
                }
            }
            fprintf(mf,"%d %d %d ",rd,gr,bl);
        }
    fclose(mf);
    return(0);
}


double _Complex fixp(arry T)
{
    double _Complex z,tr,tr2,a,b,c,d;
    double _Complex k;
    a=T[0][0];
    b=T[0][1];
    c=T[1][0];
    d=T[1][1];
    tr=a+d;
    tr2=a*a+2*b*c+d*d;
    k=((tr+csqrt(tr2-4))/2)*((tr+csqrt(tr2-4))/2);
    if (cabs(k)>1)
        z=(-(d-a)+csqrt((d-a)*(d-a)+4*b*c))/(2*c);
    else
        z=(-(d-a)-csqrt((d-a)*(d-a)+4*b*c))/(2*c);
    return(z);
}


void mult(arry a,arry b,arry c)
{
    int i,j;
    for (i=0;i<2;i++)
        for (j=0;j<2;j++)
            c[i][j]=a[i][0]*b[0][j]+a[i][1]*b[1][j];
}

void inv(arry a, arry c)
{
    double _Complex d;
    d=a[0][0]*a[1][1]-a[0][1]*a[1][0];
    c[0][0]=a[1][1]/d;
    c[0][1]=-a[0][1]/d;
    c[1][0]=-a[1][0]/d;
    c[1][1]=a[0][0]/d;
}

void asgn(arry a, arry c)
{
    c[0][0]=a[0][0];
    c[0][1]=a[0][1];
    c[1][0]=a[1][0];
    c[1][1]=a[1][1];
}