function outfl=depthfirstcom(A,B,ep,levmax,scal,sav,spec,siz,iterate,tcomm,typ,offs,fsn)
% A,B are traces of the Mobius transforms generating a,b,a^-1,b^-1
% ep=epsilon in the tree search using commutator attracting fixed pts
% levmax is the maximum depth of the tree search (10000 if set to 0)
% scal is the scaling factor for depicting the image
% sav 1,3,5 saves the file as a png. if sav>=2 we also have a program timer
% sav>2 prints dots when the maximum level reached
% sav=-1 test mode - exits on completion outfl=1 or on reaching levmax outfl=0 
% the timer reports tabs(1-4) as the numbers cycle through branches
% you can either produce a black line plot or a coloured image
% iterate = -2 plot n>0 colour image  by nth iterate, -1 by final level, 0 black
% spec=0 standard method
% spec=1 special words used also on fixed pts of a,b,a^-1,b^-1
% this refines whorls with necks and when A=2 and B=2 closes the gasket gaps
% spec=2,3 as 0,1 but for  non-free groups using automatic group table.
% Add 4 to select the opposite tab root for typ 0,1,3-4,6-9 otherwise 0->2 for typ 2,4-5.
% To include special words from the subfunction specialwords() add 8 to spec
% typ allows for several normalizations (see line 100 default=0 grandma)
% offs moves the image centre (default=0)
% FSA is used to remove null words when (aba^-1b^-1)^2=I
% siz is the image size default 1100x1100
% if you add a tenth parameter tab you can generate non-free limit sets
% tab is the trace of ab used in the advanced grandma algorithm
% to test full exploration of the tree uncomment 102-107, with levmax=4

%depthfirst(2,2,0.001,1000,1,0); %Standard option=0
%depthfirst(2,2,0.001,1000,1,0,1); %spec=1 closing gaps
%depthfirst(1.87+0.1i,1.87-0.1i,0.001,1000,1/2.8,0,0,1100,3); %3rd iterate
%depthfirst(1.87+0.1i,1.87-0.1i,0.001,1000,1/2.8,0,0);
%depthfirst(1.87+0.1i,1.87-0.1i,0.01,1000,1/2.8,0,1); special words

%Example 11.1 non-free group with relations (aba^-1b^-1)^2=I
%depthfirst(1.924781-0.047529i,2,0.005,1000,2,1,2,1100,-2,1.9248-1.4617i);
%depthfirst(1.9247306-0.0449408i, 1.91+0.2i,0.005,3000,1/6,1,2,1100,-2, 1.8569-1.2426i);
%depthfirst(2,2,0.005,1000,2,2,2,1100,-2,2.0000-1.4142i);
%or depthfirst(2,2,0.005,1000,2,2,2,1100,-2,nonfree(2,2,0,0));

%Special words example 8.16 with aaB parabolic.
%the function specialwords can be reprogrammed to handle examples like
%the double cusps where longer expressions like 2/5 where aaaBaaB is parabolic
%depthfirst(2+i,3,0.005,3000,1,0,8);

%Current explorations:
%depthfirstcom(1,1.96+0.5i,0.001,1000,0.2,2,2,1100,-2,-1,0,0,'FSA/a^3,abAB^3');
%depthfirstcom(2,2,0.001,1000,1.2,2,2,1100,-2,-1,0,0,'FSA/abAB^3');
%depthfirstcom(2,2,0.0001,1000,0.3,2,2,1100,-2,-1,8,-2.2i,'FSA/abAB^3');
%depthfirstcom(2,2,0.001,1000,0.3,2,2,1100,-2,0,0);
%depthfirstcom(2,2,0.0001,1000,0.5,2,2,1100,-2,0,8,1.4i);
%depthfirstcom(2,2,0.001,1000,0.5,2,2,1100,-2,1,0,0);
%depthfirstcom(2,2,0.001,1000,0.15,2,2,1100,-2,1,8,-1i);
%depthfirstcom(2,2,0.0001,1000,0.18,2,2,1100,-2,-2*cos(pi/7),8,4i,'FSA/abAB^7');
%depthfirstcom(1.98-0.05i,2,0.0005,1000,0.8,4,2,1100,-2,1,8,1.5i);
%depthfirstcom(1.98-0.05i,1.98+0.05i,0.0005,1000,0.8,4,2,1100,-2,1,8,1.5i);
%depthfirstcom(1.95+0.05i,1.2495-1.7368i,0.003,40,0.18,2,2,1100,-2,-2*cos(2*pi/7),6,-1+1.6i,'FSA/abAB^7');
%depthfirstcom(1.6088+0.6664i,-1.6088+0.6664i,0.003,20,0.036,2,6,1100,-2,0,6,0);
%depthfirstcom(1.6088+0.6664i,-1.6088+0.6664i,0.005,20,0.136,2,2,1100,-2,0,6,0);
%depthfirstcom(1.95-0.05i,1.92-0.1i,0.005,1000,0.102,2,2,1100,0,1,8,0);
%depthfirstcom(1.93+0.05i,1,0.003,20,0.018,2,6,1100,-2,-2*cos(2*pi/7),0,0,'FSA/b^3,abAB^7');
%depthfirstcom(1.93+0.05i,1,0.000003,20,1.4,2,2,1100,-2,-2*cos(2*pi/7),8,1.1i,'FSA/b^3,abAB^7');
%depthfirstcom(1.93+0.05i,1,0.0003,20,0.4,2,6,1100,-2,-2*cos(2*pi/7),8,1.1i,'FSA/b^3,abAB^7');

if nargin<10
    tcomm=-2;
end
if nargin<7
    spec=0;
end
if spec>=8
    specwords=1;
else
    specwords=0;
end
%spec=mod(spec,4);
opt=spec;
ta=A;
tb=B;
p=-ta*tb;
q=ta^2+tb^2-tcomm-2;
if opt>=4
    tab=(-p+sqrt(p^2-4*q))/2;
else
    tab=(-p-sqrt(p^2-4*q))/2;
end
%opt=mod(opt,4);
%[a,b]=grandma(A,B,0);
if nargin<12
    offs=0;
end
if nargin<11
    typ=0;
end
if nargin<9
    iterate=-2;
    colfac=1;
end
if iterate==-2
    colfac=0;
else
    colfac=1;
end

% colfac=1; %Set to 0 for a black and white image 1 for colour-coded
% iterate=1; %Which iterate to colour code by -1 colours by the final level

ext=0; %set to 1 to retun if image out of range (large or chaotic)
dotc=0;
outfl=true;
if nargin<8
    siz=1100;
end
if nargin<6
    sav=0;
end
if nargin<5
    scal=1;
end
ep=ep/scal;
tic;
clf;
hold on
M=256*ones(siz);
if mod(spec,8)>=4
    posrt=1;
else
    posrt=0;
end
%[a,b]=polar(A,B,posrt,tab);

% if nargin<10
%     [a,b]=grandma(A,B,posrt,0);
% else
%     [a,b]=grandma(A,B,posrt,2,tab);
% end
if opt>=4
    opt2=1;
else
    opt2=0;
end
opt=mod(opt,4);
switch typ
    case 1
        [a,b]=polar(A,B,opt2,opt,tab);
    case 2
        [a,b]=symmetric(A,B,opt2,opt,tab,tcomm);
    case 3
        [a,b]=canonical(A,B,1,opt,tab,tcomm);
    case 4
        [a,b]=canoncomm(A,B,opt2,opt,tab,tcomm);
    case 5 %recreate grandma
        [a,b,c]=symmetric(A,B,opt2,opt,tab,tcomm);
        a=b;
        b=c*b^2;
        a=a^-1;  
    case {6,7,8,9} 
%         [a,b,c]=symmetric(A,B,opt2,opt,tab,tcomm);
        [a,b]=grandma(A,B,opt2,2,tab);
        h1=a^-1;
        h2=b^-1;
        a=h1*h2;
        b=h1;
        switch typ 
            case 6 %self-inverse 1
                c=[1 1;-1 1];
            case 7 %si 2
                c=0.5*[1+1i -1+1i;1+1i 1-1i];
            case 8 %asi 1
                c=0.5*[1+1i -1-1i;-1+1i -1+1i];
            case 9 %asi 2
                c=[1+1i 1-1i;1-1i 1+1i];        
        end
        d=c^-1;
        a=c*a*d;
        b=c*b*d;
        
        
    otherwise
        [a,b]=grandma(A,B,opt2,2,tab);
end
gens=cell(1,4);
gens{1}=a;
gens{2}=b;
gens{3}=a^-1;
gens{4}=b^-1;
if levmax==0;
    levmax=10000;
end
word=cell(1,levmax);
tags=zeros(1,levmax);
state=zeros(1,levmax);
%Indra's Pearls FSA for commutator trace 0
if nargin<13
    FSA=[2 3 4 5;2 6 0 5;8 3 9 0;0 3 4 7;10 0 11 5;8 3 12 0;13 0 11 5;2 6 0 14;0 3 4 15;2 16 0 5;0 17 4 5;0 3 4 18;2 19 0 5;10 0 0 5;0 0 11 5;8 3 0 0;0 3 9 0;0 0 11 5;8 3 0 0];
else
    load(fsn);
end
    %MAF FSA Generated using [(a*b*A*B)^2,IdWord] with generator order [a,b,A,B] Also works with comm trace 0
%FSA=[2,3,4,5;2,6,0,5;7,3,8,0;0,9,4,5;10,0,11,5;7,3,12,0;2,6,0,13;0,9,4,14;15,3,8,0;2,16,0,5;0,17,4,5;0,9,4,18;10,0,0,5;19,0,11,5;20,6,0,0;7,6,0,0;0,3,8,0;0,0,11,5;21,0,0,5;2,6,0,22;2,23,0,5;10,0,24,5;7,3,25,0;0,0,4,5;0,9,8,0];
%Table replicating std exs.
%FSA=[2 3 4 5;2 3 0 5;2 3 4 0;0 3 4 5;2 0 4 5];
%spec=mod(spec,4);
stfl=true;
oldpoint=0; 
if ~specwords
    fix=zeros(3,4);
    for i=1:4
        fix(1,i)=fixp(gens{mymod(i-1)}*gens{mymod(i-2)}*gens{mymod(i-3)}*gens{mymod(i-4)});
        fix(2,i)=fixp(gens{i});
        fix(3,i)=fixp(gens{mymod(i+1)}*gens{mymod(i+2)}*gens{mymod(i+3)}*gens{mymod(i+4)});
    end
else
    [numfp,fix]=specialwords1(a,b);
    points=zeros(max(numfp));
end
%spec=mod(spec,4);
%stpt=0;
lev=1;
tags(1)=1;
state(1)=1;
word{1}=gens{1};
dotfl=false;
while ~(lev==0&&tags(1)==2)
    fl=false;
    if spec==2||spec==3||spec==6||spec==7
        if state(lev)==0
            fl=true;
        end
    end
    while ~fl
         plt=false;
         switch spec
             case {1,3,5,7}
                newpoint=mob_on_point(word{lev},fix(1,tags(lev))); %branch termination?
                oldpoint=mob_on_point(word{lev},fix(3,tags(lev))); %branch termination?
                midpoint=mob_on_point(word{lev},fix(2,tags(lev)));
                if ((lev==levmax) || (abs(newpoint-midpoint)<ep && abs(oldpoint-midpoint)<ep))
                    plt=true;
                end
             case {0,2,4,6}
               newpoint=mob_on_point(word{lev},fix(1,tags(lev))); %branch termination?
               oldpoint=mob_on_point(word{lev},fix(3,tags(lev))); %branch termination?
               if ((lev==levmax) || (abs(newpoint-oldpoint)<ep))
                    plt=true;
               end
             case 8
               pts=numfp(tags(lev));
               fl4=true;
               pts=numfp(tags(lev));
               points(1)=mob_on_point(word{lev},fix(1,tags(lev)));
               for i=2:pts
                   points(i)=mob_on_point(word{lev},fix(i,tags(lev)));
                   if abs(points(i-1)-points(i))>=ep
                       fl4=false;
                       break;
                   end
               end
               if lev==levmax
                   fl4=true;
               end
               if fl4
                   plt=true;
                   newpoint=points(1);
               end
          end
          if lev==levmax
             if sav>2
                fprintf('.');
                dotfl=true;
                dotc=dotc+1;
                if dotc>=200
                    dotc=0;
                    fprintf('\n');
                end
             end
             if sav==-1
                outfl=false;
                return;
             end
          end
         if spec<8
             x=scal*abs(newpoint+offs);
             if ext && x>50 %breakout to portray chaotic limit sets
                   fprintf('Exiting...\n');
                   colormap(colorcube);
                   image(255*M');
                   return
             end
             if spec<8 && x>1.6 && plt==true  %too far offscreen, so break
                 fl=true; 
                 break;            
             end
         end
         if iterate>0
             tagpos=tags(iterate);
         end
         if iterate==-1
             tagpos=tags(lev);
         end
         if iterate==0 || iterate==-2
             tagpos=0;
         end
         if lev==levmax
             fl=true;
             break;
         end
         if plt
             switch spec
                  case {1,3,5,7}
                      if A==2 && B==2
                        M=myplot(scal,M,oldpoint,midpoint,siz,tagpos,spec,offs);
                        M=myplot(scal,M,midpoint,newpoint,siz,tagpos,spec,offs);
                      else
                        M=myplot(scal,M,oldpoint,newpoint,siz,tagpos,spec,offs);
                      end
                  case {0,2,4,6}
                        M=myplot(scal,M,oldpoint,newpoint,siz,tagpos,spec,offs);
                  case 8
                        for i=2:pts
                            M=myplot(scal,M,points(i-1),points(i),siz,tagpos,spec,offs);
                        end
              end
              if ~stfl
                 if spec==2 || spec==3 || spec==6 || spec==7
                    if abs(oldpoint-stpoint)<10*ep
                    M=myplot(scal,M,oldpoint,stpoint,siz,tagpos,spec,offs);
                    end
                 end              
              end
              stpoint=newpoint;
              fl=true;
         else
            fl=false;
         end

         if fl
             if stfl
                 %stpoint=oldpoint;
                 stfl=false;
             end
            break;
         else
            lev=lev+1; %go forward
            tags(lev)=mymod(tags(lev-1)+1);
            if lev==1
                word{lev}=gens{tags(lev)};
                if spec==2 || spec==3 || spec==6 || spec==7
                    state(lev)=tags(lev);
                end
            else
                word{lev}=word{lev-1}*gens{tags(lev)};
                if spec==2 || spec==3 || spec==6 || spec==7
                    state(lev)=FSA(state(lev-1),tags(lev));
                    if state(lev)==0
                        fl=true;
                    end
                end
            end
         end
    end
    fl=true;
    while fl
        lev=lev-1; %go backward
        if lev==0 || tags(lev+1)-1~=mod(tags(lev)+2,4) %available turn
            fl=false;
        end
    end
    if (lev==0 && tags(1)==2)
       % M=myplot(scal,M,stpt,oldpoint,siz,colfac*tags(iterate),spec,offs);
        if dotfl
            fprintf('\n');
            dotfl=false;
        end
        if sav>=2
            fprintf('%d %d %d %d %4.4f\n', tags(1),tags(2),tags(3),tags(4),toc);
        end
        toc;
        break; %Finished - exit routine       
    end
    lev=lev+1;
    tags(lev)=mymod(tags(lev)-1); %turn and go forward
    if sav>=2
         if lev<4 %progress clock for long iterations
            if dotfl
                fprintf('\n');
                dotfl=false;
            end
            fprintf('%d %d %d %d %4.4f\n', tags(1),tags(2),tags(3),tags(4),toc);
         end
    end
    if lev==1
        word{1}=gens{tags(1)};
        %state(1)=tags(1);
    else
        word{lev}=word{lev-1}*gens{tags(lev)};
        if spec==2 || spec==3 || spec==6 || spec==7
            state(lev)=FSA(state(lev-1),tags(lev));
%             if state(lev)==0
%                 fl=true;
%             end
        end
    end
end
s=size(M); % Plot the matrix and save a png file
tit=[num2str(A),', ',num2str(B),', ',num2str(ep),', ',num2str(levmax)];
if spec==2 || spec==3 || spec==6 || spec==7
    tit=[tit,', ',num2str(tab)];
end
z=zeros(1,s(1)*s(2));
k=1;
sc11=11/10;
if ~colfac   
    for i=1:s(1)
        for j=1:s(2)
            if M(i,j)<255
                z(k)=((i-s(1)/2)*2/s(1)+1i*(j-s(2)/2)*2/s(2))*sc11;
                k=k+1;
            end
        end
    end
    plot(z(1:k-1),'.','MarkerSize',1,'Color','k');
    axis([-sc11 sc11 -sc11 sc11]);
    text(-1,1.05,tit);
else
    image(M');
    colormap(colorcube(256));
    sz=size(M');
    axis([0 s(2) 0 s(1)]);
end
if sav==1 || sav==3 || sav==5
%     if colfac==0
%        imwrite(255*M',[tit,'.png']);
%     else
       imwrite(M',colorcube(256),[tit,'.png']);
%     end
    %save([tit2,'.mat'],'M');
end

function n=mymod(m)
n=mod(m,4);
if n==0
    n=4;
end

function M=myplot(sc,M,o,n,siz,c,spec,offs) %Plots a scaled image in M
o=o+offs;
n=n+offs;
siz=siz/2;
sizs=10/11*siz;
fl=true;
big=10/sc;
if (abs(o-offs))>big || abs(n-offs)>big %added to avoid plotting line offscreen
    fl=false;
end
switch c
    case 0
        co=241; %black
    case 4
        co=186; %magenta
    case 1
        co=233; %blue
    case 2
        co=208; %red
    case 3
        co=165; %cyan
        end
% if spec==2 && abs(n-o)>0.1/sc %added to remove four extraneous lines
%     fl=false;
% end
if fl
    if abs(imag(n-o))>abs(real(n-o))
        stps=max(ceil(abs(sc*imag(n-o)*siz)),2);
        zs=linspace(sc*o,sc*n,stps);
    else
        stps=max(ceil(abs(sc*real(n-o)*siz)),2);
        zs=linspace(sc*o,sc*n,stps);
    end
    for i=1:length(zs)
        if ~isnan(zs(i))
            if max(abs(real(zs(i))),abs(imag(zs(i))))<1
               M(siz-floor(sizs*real(zs(i))),siz-floor(sizs*imag(zs(i))))=co;
            end
        end
    end
end
        
function zo=mob_on_point(T,z) %Mobius transforms a point
%D=struct();
a=T(1,1);
b=T(1,2);
c=T(2,1);
d=T(2,2);
zo=(a*z+b)/(c*z+d);

function z=fixp(T) %Finds an attracting fixed point of a transformation
a=T(1,1);
b=T(1,2);
c=T(2,1);
d=T(2,2);
T2=T^2;
tr=T(1,1)+T(2,2);
tr2=T2(1,1)+T2(2,2);
k=((tr+sqrt(tr2-4))/2)^2;
if abs(k)>1
    z=(-(d-a)+((d-a)^2+4*b*c))^(1/2)/(2*c);
 else
    z=(-(d-a)-((d-a)^2+4*b*c))^(1/2)/(2*c);
 end

% function [a,b]=grandma(ta,tb,posroot,opt,tab)
% if nargin<5
%     opt=mod(opt,2);
% end
% if nargin<4
%     opt=0;
% end
% if nargin<3
%     posroot=false;
% end
% if opt<2 %Calculate the trace Tab from the grandfather identity
%     p=-ta*tb;
%     q=ta^2+tb^2;
%     if posroot
%         tab=(-p+(p^2-4*q)^0.5)/2;
%     else
%         tab=(-p-(p^2-4*q)^0.5)/2;
%     end
%     z0=((tab-2)*tb)/(tb*tab-2*ta+2*1i*tab);
%     if opt==0 %Grandma's original recipe
%     ab=[tab/2 (tab-2)/(2*z0);(tab+2)*z0/2 tab/2];
%     b=[(tb-2*1i)/2 tb/2;tb/2 (tb+2*1i)/2];
%     a=ab*b^-1;
%     end
%     if opt==1 %Jorgensen's recipe
%     a=[ta-tb/tab ta/tab/tab;ta tb/tab];
%     b=[tb-ta/tab -tb/tab/tab;-tb ta/tab];
%     end
%  else
%     tC=ta^2+tb^2+tab^2-ta*tb*tab-2; %Grandma's three parameter double alarm
%    
%     Q=sqrt(2-tC);
%     if abs(tC+1i*Q*sqrt(tC+2))>=2+1e-15 %e-15 added to avoid Ta = Tb =2 error
%         R=sqrt(tC+2);
%     else
%         R=-sqrt(tC+2);
%     end
%     %R=-R;
%     zo=(tab-2)*(tb-R)/(tb*tab-2*ta+1i*Q*tab);
%     a=[ta/2 (ta*tab-2*tb+2i*Q)/((2*tab+4)*zo);(ta*tab-2*tb-2i*Q)*zo/(2*tab-4) ta/2];
%     b=[(tb-1i*Q)/2 (tb*tab-2*ta-1i*Q*tab)/((2*tab+4)*zo);(tb*tab-2*ta+1i*Q*tab)*zo/(2*tab-4) (tb+1i*Q)/2];
% end

function [numfp, fix]=specialwords1(a,b)
A=a^-1;
B=b^-1;
numfp=zeros(1,4);
for i=1:4
    switch i
    case {1,3}
        numfp(i)=4;
    case{2,4}
        numfp(i)=3;
    end
end
fix=zeros(4,4);
fix(1,1)=fixp(b*A*B*a);
fix(2,1)=fixp(a*B*a);
fix(3,1)=fixp(B*a*a);
fix(4,1)=fixp(B*A*b*a);
fix(1,2)=fixp(A*B*a*b);
fix(2,2)=fixp(A*A*b);
fix(3,2)=fixp(a*B*A*b);
fix(1,3)=fixp(B*a*b*A);
fix(2,3)=fixp(A*b*A);
fix(3,3)=fixp(b*A*A);
fix(4,3)=fixp(b*a*B*A);
fix(1,4)=fixp(a*b*A*B);
fix(2,4)=fixp(a*a*B);
fix(3,4)=fixp(A*b*a*B);