function [tab,tcomm,a,b]=nonfree(A,B,opt,scal,tcomm)
% Generates the trace Tab for a non-free group with TabAB=0. 
%Also allows arbitrary TabAB if tcomm is set to a value.
%opt= 0/1 give either quare root
%if scal?0 gives scaled stochastic portrait
%e.g. tab=nonfree(2,2,0,1/3) %Non-free gasket.
%e.g. tab=nonfree(2,2,0,3/2,-1) %arbitrary tcomm
%e.g. tab=nonfree(2,2,0,0) %No plot good for using with depthfirst
if nargin< 5
    tcomm=0;
end
if nargin< 4
    scal=1;
end
ta=A;
tb=B;
p=-ta*tb;
q=ta^2+tb^2-tcomm-2;
if opt
    tab=(-p+sqrt(p^2-4*q))/2;
else
    tab=(-p-sqrt(p^2-4*q))/2;
end
[a,b]=grandma(A,B,0,2,tab);
comm=a*b*a^-1*b^-1;
tcomm=comm(1,1)+comm(2,2);
if scal~=0
    qFstochastic(A,B,40,1,1100,scal,2,2,tab);
end

function [a,b]=grandma(ta,tb,posroot,opt,tab)
if nargin<5
    opt=mod(opt,2);
end
if nargin<4
    opt=0;
end
if nargin<3
    posroot=false;
end
if opt<2 %Calculate thre trace Tab from the grandfather identity
    p=-ta*tb;
    q=ta^2+tb^2;
    if posroot
        tab=(-p+(p^2-4*q)^0.5)/2;
    else
        tab=(-p-(p^2-4*q)^0.5)/2;
    end
    z0=((tab-2)*tb)/(tb*tab-2*ta+2*1i*tab);
    if opt==0 %Grandma's original recipe
    ab=[tab/2 (tab-2)/(2*z0);(tab+2)*z0/2 tab/2];
    b=[(tb-2*1i)/2 tb/2;tb/2 (tb+2*1i)/2];
    a=ab*b^-1;
    end
    if opt==1 %Jorgensen's recipe
    a=[ta-tb/tab ta/tab/tab;ta tb/tab];
    b=[tb-ta/tab -tb/tab/tab;-tb ta/tab];
    end
 else
    tC=ta^2+tb^2+tab^2-ta*tb*tab-2; %Grandma's three parameter double alarm
    Q=sqrt(2-tC);
    if abs(tC+1i*Q*sqrt(tC+2))>=2
        R=sqrt(tC+2);
    else
        R=-sqrt(tC+2);
    end
    zo=(tab-2)*(tb-R)/(tb*tab-2*ta+1i*Q*tab);
    a=[ta/2 (ta*tab-2*tb+2i*Q)/((2*tab+4)*zo);(ta*tab-2*tb-2i*Q)*zo/(2*tab-4) ta/2];
    b=[(tb-1i*Q)/2 (tb*tab-2*ta-1i*Q*tab)/((2*tab+4)*zo);(tb*tab-2*ta+1i*Q*tab)*zo/(2*tab-4) (tb+1i*Q)/2];
end
