function qFmaskit(u,v,K,M,opt,resol,cycs)
%u,v real and imaginary parameter 1 parameter 2=2
%K, M sigmoid division curve slope and exponent 
%opt=1 draw dividing curve to define iterations
%resol gives images resolution cycs no of iterations
%opt=odd draw dividing curve

%e.g. qFmaskit(1.95,0.07,0.2,1.5,1)
%e.g. qFmaskit(1.74,0.24,0.28,2.5,1)
%e.g. qFmaskit(1.68,0.34,0.2,3.5,1,1000,100)
%e.g. qFmaskit(1.65,0.5,0.18,4.3,1)
%e.g. qFmaskit(1.867,0.064,0.3,3.5,1)
%e.g. qFmaskit(1.64,0.8765,0.18,2,1)
%e.g. qFmaskit(1.912,1/30,0.26,4.9,1)
%e.g. qFmaskit(1.6915,5/9+0.0255,0.105,6.2,0,1000,1000)

if nargin<7
    cycs=100;
end
if nargin<6
    resol=1000;
end
clf;

its=2*resol+1;
scal=2;
Mt=ones(its,resol);
N=-10;
%K=0.1;
%M=1/2;
e=0.0001;
xb=zeros(1,3);
yb=zeros(1,3);
for i=1:3
    xb(i)=i;
    yb(i)=i;
end
%u=1.95;
%v=0.07;
% u=2;
% v=0;
t=u+1i*v;

for i=1:its
    for j=1:resol
        x=(i-resol)*scal/resol;
        y=(j-0)*u/resol;
        for k=1:cycs
            x=mod((x+2*N-1+v*y/u),2)-1-v*y/u;
            if x>=-v/2
                f=1;
            else
                f=-1;
            end
            y2=u/2+f*K*u*(1-exp(-M*abs(x+v/2)));
          % y2=u/2;
            z=x+1i*y;
            if y<y2
                z=(t*z-1i)/-1i/z;
            else
                z=1i/(1i*z+t);
            end
            x=real(z);
            y=imag(z);
            if y<0
                Mt(i,j)=127;
                break;
            end
            if y>u
                Mt(i,j)=63;
                break;
            end
            xb(1)=xb(2);
            xb(2)=xb(3);
            xb(3)=x;
            yb(1)=yb(2);
            yb(2)=yb(3);
            yb(3)=y;
            if abs(xb(1)-xb(3))<e && abs(yb(1)-yb(3))<e
                Mt(i,j)=31;
                break;
            end
            
        end
        
    end
end
ms=['u=',num2str(u),' v=',num2str(v),' K=',num2str(K),' M=',num2str(M)];
jetimgmat(Mt',[0 0 0 0],(0:resol),0,ms,0,'hot(256)');
if mod(opt,2)==1
    hold on
    x=1:its;
    xs=zeros(1,its);
    y=zeros(1,its);
    for i=1:its
      xs=(i-resol)*scal/resol;
      if xs>=-v/2
            f=1;
      else
            f=-1;
      end
      y(i)=(u/2+f*K*u*(1-exp(-M*abs(xs+v/2))))*resol/u;
    end
    plot(x,y,'k');
    hold off
end

function jetimgmat(Mo,bounds,FreqVec,Chan,tit,cbarf,mymap)
%plots any m x n matrix in jet 256 with a frame bound if desired
%prt=1 prints actual max and min of M
%freqvec determines the y-scale
%e.g. jetimgmat(sdiff);   jetimgmat(sdiff,[5 0 50 50]);
%jetimgmat(sdiff,[5 0 50 50],0,[30:60]);
if nargin<6
   cbarf=1;
end
if nargin<7
    mymap=jet(256);
end
s=size(Mo);
if nargin==1
    ymin=1;
    ymax=s(1);
    xmin=1;
    xmax=s(2);
else
    ymin=1+bounds(1,1);
    ymax=s(1)-bounds(1,2);
    xmin=1+bounds(1,3);
    xmax=s(2)-bounds(1,4);
end
Mo=Mo(ymin:ymax,xmin:xmax);
mx=max(max(Mo));
mn=min(min(Mo));
Mo=(Mo-mn)/(mx-mn)*256;
D=Mo;
img=image(D);
if nargin>=3
    tics=linspace(1,length(FreqVec)-bounds(1,1)-bounds(1,2),4);
    labs=round(linspace(FreqVec(ymin),FreqVec(ymax),4));
    
    NSamps=s(2);
    start=1+bounds(1,3);
    finish=NSamps-1-bounds(1,4);
    len=finish-start+1;
    p=10^(floor(log(s(2))/log(10)));
    q=p*floor(s(2)/p);
    sps=10;
    mint=floor(q/sps);
    cent=NSamps/2+1-start;
    frms=cent-sps*mint:mint:cent+sps*mint;
    xlabs=-sps*mint:mint:sps*mint;
    axeAct  = get(img,'parent');
    set(axeAct, ...
    'YTick',tics, ...
    'YTickLabel',labs, ...
    'YDir','reverse', ...
    'XTick',frms, ...
    'XTickLabel',xlabs, ...
    'XDir','normal', ...
    'Box','On' ...
    );
    %myst=['M ' num2str(mx,'%4.2e') ', m ' num2str(mn,'%4.2e')];
    myst=[];
    if nargin>=4 && Chan>0
        myst=[myst,', Ch ',num2str(Chan)];
    end
    if nargin>=5
        if length(tit)>=4
            if strcmp(tit(length(tit)-3:length(tit)),'.wtf')
                tit=tit(1:length(tit)-4);
            end
        end
        myst=[tit,' ',myst];
    end
    xlabel(myst,'Parent',axeAct);
end
colormap(mymap);